"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.within = exports.installTestingLibraryFixture = exports.registerSelectorsFixture = exports.queriesFixture = void 0;
const fs_1 = require("fs");
const test_1 = require("@playwright/test");
const common_1 = require("../common");
const helpers_1 = require("./helpers");
const isAllQuery = (query) => query.includes('All');
const isNotFindQuery = (query) => !query.startsWith('find');
const queryNames = common_1.queryNames.filter(isNotFindQuery);
const queryToSelector = (query) => query.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
const queriesFixture = ({ page }, use) => __awaiter(void 0, void 0, void 0, function* () {
    const queries = queryNames.reduce((rest, query) => (Object.assign(Object.assign({}, rest), { [query]: (...args) => page.locator(`${queryToSelector(query)}=${JSON.stringify(args, helpers_1.replacer)}`) })), {});
    yield use(queries);
});
exports.queriesFixture = queriesFixture;
const within = (locator) => queryNames.reduce((rest, query) => (Object.assign(Object.assign({}, rest), { [query]: (...args) => locator.locator(`${queryToSelector(query)}=${JSON.stringify(args, helpers_1.replacer)}`) })), {});
exports.within = within;
const engine = () => ({
    query(root, selector) {
        const args = JSON.parse(selector, window.__testingLibraryReviver);
        if (isAllQuery(queryName))
            throw new Error(`PlaywrightTestingLibrary: the plural '${queryName}' was used to create this Locator`);
        // @ts-expect-error
        const result = window.TestingLibraryDom[queryName](root, ...args);
        return result;
    },
    queryAll(root, selector) {
        const testingLibrary = window.TestingLibraryDom;
        const args = JSON.parse(selector, window.__testingLibraryReviver);
        // @ts-expect-error
        const result = testingLibrary[queryName](root, ...args);
        if (!result)
            return [];
        return Array.isArray(result) ? result : [result];
    },
});
const registerSelectorsFixture = [
    ({}, use) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield Promise.all(queryNames.map((name) => __awaiter(void 0, void 0, void 0, function* () {
                return test_1.selectors.register(queryToSelector(name), `(${engine.toString().replace(/queryName/g, `"${name}"`)})()`);
            })));
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error('PlaywrightTestingLibrary: failed to register Testing Library functions\n', error);
        }
        yield use();
    }),
    { scope: 'worker', auto: true },
];
exports.registerSelectorsFixture = registerSelectorsFixture;
const installTestingLibraryFixture = [
    ({ context }, use) => __awaiter(void 0, void 0, void 0, function* () {
        const testingLibraryDomUmdScript = yield fs_1.promises.readFile(require.resolve('@testing-library/dom/dist/@testing-library/dom.umd.js'), 'utf8');
        yield context.addInitScript(`
        ${testingLibraryDomUmdScript}
        
        window.__testingLibraryReviver = ${helpers_1.reviver.toString()};
    `);
        yield use();
    }),
    { scope: 'test', auto: true },
];
exports.installTestingLibraryFixture = installTestingLibraryFixture;
//# sourceMappingURL=locator.js.map